@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  * libwnck based pager applet.
  * (C) 2001 Alexander Larsson 
@@ -30,6 +31,8 @@
 
 #include "pager.h"
 
+#include "egg-screen-help.h"
+
 /* even 16 is pretty darn dubious. */
 #define MAX_REASONABLE_ROWS 16
 #define DEFAULT_ROWS 1
@@ -102,6 +105,30 @@ pager_update (PagerData *pager)
 				 pager->display_all);
 }
 
+static WnckScreen *
+applet_get_screen (GtkWidget *applet)
+{
+	int screen_num;
+
+	if (!gtk_widget_has_screen (applet))
+		return wnck_screen_get_default ();
+
+	screen_num = gdk_screen_get_number (gtk_widget_get_screen (applet));
+
+	return wnck_screen_get (screen_num);
+}
+
+static void
+applet_realized (PanelApplet *applet,
+		 PagerData   *pager)
+{
+	WnckScreen *screen;
+
+	screen = applet_get_screen (GTK_WIDGET (applet));
+
+	wnck_pager_set_screen (WNCK_PAGER (pager->pager), screen);
+}
+
 static void
 applet_change_orient (PanelApplet       *applet,
 		      PanelAppletOrient  orient,
@@ -131,7 +158,7 @@ applet_change_orient (PanelApplet       *applet,
 }
 
 static void 
-response_cb(GtkWidget * widget,int id, gpointer data)
+response_cb(GtkWidget * widget,int id, PagerData *pager)
 {
 	if(id == GTK_RESPONSE_HELP) {
 		
@@ -146,8 +173,11 @@ response_cb(GtkWidget * widget,int id, gpointer data)
 							     GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 		}
 
-		gnome_help_display_desktop (applet_program, "workspace-switcher",
-					    "workspace-switcher", "workspacelist-prefs", &error);
+		egg_help_display_desktop_on_screen (
+			applet_program, "workspace-switcher",
+			"workspace-switcher", "workspacelist-prefs",
+			gtk_widget_get_screen (pager->applet),
+			&error);
 		if (error) {
 			GtkWidget *dialog;
 			dialog = gtk_message_dialog_new (GTK_WINDOW(widget),
@@ -162,6 +192,8 @@ response_cb(GtkWidget * widget,int id, gpointer data)
 					  NULL);
 
 			gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+			gtk_window_set_screen (GTK_WINDOW (dialog),
+					       gtk_widget_get_screen (pager->applet));
 			gtk_widget_show (dialog);
 			g_error_free (error);
 
@@ -386,7 +418,7 @@ fill_pager_applet(PanelApplet *applet)
 		break;
 	}
 
-	pager->screen = wnck_screen_get_default ();
+	pager->screen = applet_get_screen (pager->applet);
 
 	/* because the pager doesn't respond to signals at the moment */
 	wnck_screen_force_update (pager->screen);
@@ -413,6 +445,10 @@ fill_pager_applet(PanelApplet *applet)
 
 	gtk_widget_show (pager->applet);
 
+	g_signal_connect (G_OBJECT (pager->applet),
+			  "realize",
+			  G_CALLBACK (applet_realized),
+			  pager);
 	g_signal_connect (G_OBJECT (pager->applet),
 			  "change_orient",
 			  G_CALLBACK (applet_change_orient),
@@ -449,8 +485,11 @@ display_help_dialog (BonoboUIComponent *uic,
 						     GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 	}
 
-	gnome_help_display_desktop (applet_program, "workspace-switcher",
-				    "workspace-switcher", NULL, &error);
+	egg_help_display_desktop_on_screen (
+		applet_program, "workspace-switcher",
+		"workspace-switcher", NULL,
+		gtk_widget_get_screen (pager->applet),
+		&error);
 	if (error) {
 		GtkWidget *dialog;
 		dialog = gtk_message_dialog_new (NULL,
@@ -465,6 +504,8 @@ display_help_dialog (BonoboUIComponent *uic,
 				  NULL);
 
 		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (pager->applet));
 		gtk_widget_show (dialog);
 		g_error_free (error);
 
@@ -490,8 +531,9 @@ display_about_dialog (BonoboUIComponent *uic,
 	};
 	const char *translator_credits = _("translator_credits");
 
-	if (about != NULL) {
-		gtk_widget_show (about);
+	if (about) {
+		gtk_window_set_screen (GTK_WINDOW (about),
+				       gtk_widget_get_screen (pager->applet));
 		gtk_window_present (GTK_WINDOW (about));
 		return;
 	}
@@ -509,6 +551,8 @@ display_about_dialog (BonoboUIComponent *uic,
 				 pixbuf);
 	
 	gtk_window_set_wmclass (GTK_WINDOW (about), "pager", "Pager");
+	gtk_window_set_screen (GTK_WINDOW (about),
+			       gtk_widget_get_screen (pager->applet));
 
 	if (pixbuf) {
 		gtk_window_set_icon (GTK_WINDOW (about), pixbuf);
@@ -606,12 +650,8 @@ static void
 num_workspaces_value_changed (GtkSpinButton *button,
 			      PagerData       *pager)
 {
-	GConfClient* client;
-	
-	client = gconf_client_get_default ();
-
-	gconf_client_set_int (client, "/desktop/gnome/applications/window_manager/number_of_workspaces",
-			      gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (pager->num_workspaces_spin)), NULL);
+        wnck_screen_change_workspace_count (pager->screen,
+                                            gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (pager->num_workspaces_spin)));
 }
 
 static void 
@@ -620,40 +660,22 @@ workspace_name_edited (GtkCellRendererText *cell_renderer_text,
 		       const gchar         *new_text,
 		       PagerData           *pager)
 {
-	GConfClient* client;
-	GtkTreeIter iter;
-	char *str;
-	GSList *list;
+        const gint *indices;
+        WnckWorkspace *workspace;
+        GtkTreePath *p;
 
-	if (gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (pager->workspaces_store),
-						 &iter, path)) {
-		gtk_list_store_set (pager->workspaces_store,
-				    &iter,
-				    0, new_text,
-				    -1);
-	}
+        p = gtk_tree_path_new_from_string (path);
+        indices = gtk_tree_path_get_indices (p);
+        workspace = wnck_screen_get_workspace (pager->screen,
+                                               indices[0]);
+        if (workspace != NULL)
+                wnck_workspace_change_name (workspace,
+                                            new_text);
+        else
+                g_warning ("Edited name of workspace %d which no longer exists",
+                           indices[0]);
 
-	
-	list = NULL;
-	gtk_tree_model_get_iter_root (GTK_TREE_MODEL (pager->workspaces_store), &iter);
-	do {
-		gtk_tree_model_get (GTK_TREE_MODEL (pager->workspaces_store),
-				    &iter,
-				    0, &str,
-				    -1);
-		list = g_slist_prepend (list, str);
-	} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (pager->workspaces_store), &iter));
-	
-	list = g_slist_reverse (list);
-
-	client = gconf_client_get_default ();
-	gconf_client_set_list (client,  "/desktop/gnome/applications/window_manager/workspace_names",
-			       GCONF_VALUE_STRING,
-			       list,
-			       NULL);
-
-	g_slist_foreach (list, (GFunc)g_free, NULL);
-	g_slist_free (list);
+        gtk_tree_path_free (p);
 }
 
 static gboolean
@@ -794,6 +816,8 @@ display_properties_dialog (BonoboUIComponent *uic,
 		g_object_unref (G_OBJECT (xml));
 	}
 
+	gtk_window_set_screen (GTK_WINDOW (pager->properties_dialog),
+			       gtk_widget_get_screen (pager->applet));
 	gtk_window_present (GTK_WINDOW (pager->properties_dialog));
 }
 
