@@ -28,6 +28,8 @@ typedef struct {
 	GnomeDesktopItem *revert_ditem;
 
 	GtkWidget         *prop_dialog;
+
+	gulong             icon_changed_signal;
 } Launcher;
 
 /* If launchers are loaded from ANYWHERE but the normal panel
@@ -63,7 +65,8 @@ Launcher *	load_launcher_applet_from_info_url(const char *name,
 						   int pos,
 						   gboolean exactpos);
 
-void		launcher_properties		(Launcher  *launcher);
+void		launcher_properties		(Launcher  *launcher,
+						 GdkScreen *screen);
 
 void            launcher_save_to_gconf          (Launcher   *launcher,
 						 const char *gconf_key);
@@ -82,7 +85,8 @@ void		launcher_hoard			(Launcher *launcher);
 
 Launcher *	find_launcher			(const char *path);
 
-void            launcher_show_help              (Launcher *launcher);
+void            launcher_show_help              (Launcher  *launcher,
+						 GdkScreen *screen);
 
 void            launcher_properties_destroy     (Launcher *launcher);
 
